/*
 * Copyright 2017 Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL VMWARE AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "tu_private.h"

#include "vk_util.h"

/* Convert the VK_USE_PLATFORM_* defines to booleans */
#ifdef VK_USE_PLATFORM_ANDROID_KHR
#   undef VK_USE_PLATFORM_ANDROID_KHR
#   define VK_USE_PLATFORM_ANDROID_KHR true
#else
#   define VK_USE_PLATFORM_ANDROID_KHR false
#endif
#ifdef VK_USE_PLATFORM_WAYLAND_KHR
#   undef VK_USE_PLATFORM_WAYLAND_KHR
#   define VK_USE_PLATFORM_WAYLAND_KHR true
#else
#   define VK_USE_PLATFORM_WAYLAND_KHR false
#endif
#ifdef VK_USE_PLATFORM_XCB_KHR
#   undef VK_USE_PLATFORM_XCB_KHR
#   define VK_USE_PLATFORM_XCB_KHR true
#else
#   define VK_USE_PLATFORM_XCB_KHR false
#endif
#ifdef VK_USE_PLATFORM_XLIB_KHR
#   undef VK_USE_PLATFORM_XLIB_KHR
#   define VK_USE_PLATFORM_XLIB_KHR true
#else
#   define VK_USE_PLATFORM_XLIB_KHR false
#endif
#ifdef VK_USE_PLATFORM_DISPLAY_KHR
#   undef VK_USE_PLATFORM_DISPLAY_KHR
#   define VK_USE_PLATFORM_DISPLAY_KHR true
#else
#   define VK_USE_PLATFORM_DISPLAY_KHR false
#endif
#ifdef VK_USE_PLATFORM_XLIB_XRANDR_EXT
#   undef VK_USE_PLATFORM_XLIB_XRANDR_EXT
#   define VK_USE_PLATFORM_XLIB_XRANDR_EXT true
#else
#   define VK_USE_PLATFORM_XLIB_XRANDR_EXT false
#endif

/* And ANDROID too */
#ifdef ANDROID
#   undef ANDROID
#   define ANDROID true
#else
#   define ANDROID false
#endif

#define TU_HAS_SURFACE (VK_USE_PLATFORM_WAYLAND_KHR ||                          VK_USE_PLATFORM_XCB_KHR ||                          VK_USE_PLATFORM_XLIB_KHR ||                          VK_USE_PLATFORM_DISPLAY_KHR)


const VkExtensionProperties tu_instance_extensions[TU_INSTANCE_EXTENSION_COUNT] = {
   {"VK_KHR_get_display_properties2", 1},
   {"VK_KHR_get_physical_device_properties2", 1},
   {"VK_KHR_get_surface_capabilities2", 1},
   {"VK_KHR_surface", 25},
   {"VK_KHR_wayland_surface", 6},
   {"VK_KHR_xcb_surface", 6},
   {"VK_KHR_xlib_surface", 6},
   {"VK_KHR_display", 23},
   {"VK_EXT_direct_mode_display", 1},
   {"VK_EXT_acquire_xlib_display", 1},
   {"VK_EXT_display_surface_counter", 1},
   {"VK_EXT_debug_report", 9},
   {"VK_KHR_external_memory_capabilities", 1},
};

const VkExtensionProperties tu_device_extensions[TU_DEVICE_EXTENSION_COUNT] = {
   {"VK_KHR_bind_memory2", 1},
   {"VK_KHR_create_renderpass2", 1},
   {"VK_KHR_dedicated_allocation", 1},
   {"VK_KHR_get_memory_requirements2", 1},
   {"VK_KHR_maintenance1", 1},
   {"VK_KHR_maintenance2", 1},
   {"VK_KHR_maintenance3", 1},
   {"VK_KHR_sampler_mirror_clamp_to_edge", 1},
   {"VK_KHR_swapchain", 68},
   {"VK_EXT_display_control", 1},
   {"VK_KHR_external_memory", 1},
   {"VK_KHR_external_memory_fd", 1},
   {"VK_EXT_external_memory_dma_buf", 1},
   {"VK_EXT_image_drm_format_modifier", 1},
   {"VK_EXT_sample_locations", 1},
   {"VK_EXT_sampler_filter_minmax", 1},
   {"VK_EXT_transform_feedback", 1},
   {"VK_ANDROID_native_buffer", 1},
   {"VK_KHR_external_semaphore_fd", 1},
   {"VK_KHR_external_fence_fd", 1},
   {"VK_IMG_filter_cubic", 1},
   {"VK_EXT_filter_cubic", 1},
};

const struct tu_instance_extension_table tu_supported_instance_extensions = {
   .KHR_get_display_properties2 = VK_USE_PLATFORM_DISPLAY_KHR,
   .KHR_get_physical_device_properties2 = true,
   .KHR_get_surface_capabilities2 = TU_HAS_SURFACE,
   .KHR_surface = TU_HAS_SURFACE,
   .KHR_wayland_surface = VK_USE_PLATFORM_WAYLAND_KHR,
   .KHR_xcb_surface = VK_USE_PLATFORM_XCB_KHR,
   .KHR_xlib_surface = VK_USE_PLATFORM_XLIB_KHR,
   .KHR_display = VK_USE_PLATFORM_DISPLAY_KHR,
   .EXT_direct_mode_display = VK_USE_PLATFORM_DISPLAY_KHR,
   .EXT_acquire_xlib_display = VK_USE_PLATFORM_XLIB_XRANDR_EXT,
   .EXT_display_surface_counter = VK_USE_PLATFORM_DISPLAY_KHR,
   .EXT_debug_report = true,
   .KHR_external_memory_capabilities = true,
};

void tu_fill_device_extension_table(const struct tu_physical_device *device,
                                      struct tu_device_extension_table* table)
{
   table->KHR_bind_memory2 = true;
   table->KHR_create_renderpass2 = true;
   table->KHR_dedicated_allocation = true;
   table->KHR_get_memory_requirements2 = true;
   table->KHR_maintenance1 = true;
   table->KHR_maintenance2 = true;
   table->KHR_maintenance3 = true;
   table->KHR_sampler_mirror_clamp_to_edge = true;
   table->KHR_swapchain = TU_HAS_SURFACE;
   table->EXT_display_control = VK_USE_PLATFORM_DISPLAY_KHR;
   table->KHR_external_memory = true;
   table->KHR_external_memory_fd = true;
   table->EXT_external_memory_dma_buf = true;
   table->EXT_image_drm_format_modifier = false;
   table->EXT_sample_locations = device->gpu_id == 650;
   table->EXT_sampler_filter_minmax = true;
   table->EXT_transform_feedback = true;
   table->ANDROID_native_buffer = true;
   table->KHR_external_semaphore_fd = true;
   table->KHR_external_fence_fd = true;
   table->IMG_filter_cubic = device->gpu_id == 650;
   table->EXT_filter_cubic = device->gpu_id == 650;
}

VkResult tu_EnumerateInstanceVersion(
    uint32_t*                                   pApiVersion)
{
    *pApiVersion = VK_MAKE_VERSION(1, 2, 131);
    return VK_SUCCESS;
}

uint32_t
tu_physical_device_api_version(struct tu_physical_device *dev)
{
    return VK_MAKE_VERSION(1, 1, 82);
}
