/*
 * Copyright 2017 Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sub license, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice (including the
 * next paragraph) shall be included in all copies or substantial portions
 * of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT.
 * IN NO EVENT SHALL VMWARE AND/OR ITS SUPPLIERS BE LIABLE FOR
 * ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef TU_EXTENSIONS_H
#define TU_EXTENSIONS_H

enum {
   TU_INSTANCE_EXTENSION_COUNT = 13,
   TU_DEVICE_EXTENSION_COUNT = 22,
};

struct tu_instance_extension_table {
   union {
      bool extensions[TU_INSTANCE_EXTENSION_COUNT];
      struct {
        bool KHR_get_display_properties2;
        bool KHR_get_physical_device_properties2;
        bool KHR_get_surface_capabilities2;
        bool KHR_surface;
        bool KHR_wayland_surface;
        bool KHR_xcb_surface;
        bool KHR_xlib_surface;
        bool KHR_display;
        bool EXT_direct_mode_display;
        bool EXT_acquire_xlib_display;
        bool EXT_display_surface_counter;
        bool EXT_debug_report;
        bool KHR_external_memory_capabilities;
      };
   };
};

struct tu_device_extension_table {
   union {
      bool extensions[TU_DEVICE_EXTENSION_COUNT];
      struct {
        bool KHR_bind_memory2;
        bool KHR_create_renderpass2;
        bool KHR_dedicated_allocation;
        bool KHR_get_memory_requirements2;
        bool KHR_maintenance1;
        bool KHR_maintenance2;
        bool KHR_maintenance3;
        bool KHR_sampler_mirror_clamp_to_edge;
        bool KHR_swapchain;
        bool EXT_display_control;
        bool KHR_external_memory;
        bool KHR_external_memory_fd;
        bool EXT_external_memory_dma_buf;
        bool EXT_image_drm_format_modifier;
        bool EXT_sample_locations;
        bool EXT_sampler_filter_minmax;
        bool EXT_transform_feedback;
        bool ANDROID_native_buffer;
        bool KHR_external_semaphore_fd;
        bool KHR_external_fence_fd;
        bool IMG_filter_cubic;
        bool EXT_filter_cubic;
      };
   };
};

extern const VkExtensionProperties tu_instance_extensions[TU_INSTANCE_EXTENSION_COUNT];
extern const VkExtensionProperties tu_device_extensions[TU_DEVICE_EXTENSION_COUNT];
extern const struct tu_instance_extension_table tu_supported_instance_extensions;


struct tu_physical_device;

void tu_fill_device_extension_table(const struct tu_physical_device *device,
                                      struct tu_device_extension_table* table);
#endif
